#! /usr/bin/env python2.5
# -*- coding: utf8 -*-

# Simple python script that add entry from an imap box
# Author : Benjamin Petit <benjamin@foogitiff.net> 2010

#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import imaplib 
import ssmtplib
import smtplib
import email
import mimetools
import tempfile
import os
import StringIO
import ConfigParser
import time
from email.mime.text import MIMEText


class Mail2nb:

   def __init__(self, conf_file="mail2nb.conf"):
      # read the config file
      self.read_config(conf_file)

   def read_config(self,conf_file):
      config = ConfigParser.RawConfigParser()
      config.read(conf_file)
      # general
      self.delay        = config.getint("General","delay")
      self.mail_from    = config.get("General","mail_from")
      self.mail_me      = config.get("General","mail_me")
      self.mail_subject = config.get("General","mail_subject")
      # IMAP
      self.imap_url     = config.get("IMAP","url")
      self.imap_port    = config.getint("IMAP","port")
      self.imap_ssl     = config.getboolean("IMAP","use_ssl")
      self.imap_user    = config.get("IMAP","user")
      self.imap_passwd  = config.get("IMAP","passwd")
      # SMTP
      self.smtp_enabled = config.getboolean("SMTP","enabled")
      self.smtp_url     = config.get("SMTP","url")
      self.smtp_port    = config.getint("SMTP","port")
      self.smtp_ssl     = config.getboolean("SMTP","use_ssl")
      self.smtp_user    = config.get("SMTP","user")
      self.smtp_passwd  = config.get("SMTP","passwd")

   def connect(self):
      self.connect_imap()
      if self.smtp_enabled:
         self.connect_smtp()

   def connect_imap(self):
      if self.imap_ssl:
         self.imap = imaplib.IMAP4_SSL(self.imap_url)
      else:
         self.imap = imaplib.IMAP4(self.imap_url)
      self.imap.login(self.imap_user,self.imap_passwd)
      self.imap.select()

   def connect_smtp(self):
      if self.smtp_ssl:
         self.smtp = ssmtplib.SMTP_SSL(self.smtp_url)
      else:
         self.smtp = ssmtplib.SMTP(self.smtp_url)
      self.smtp.login(self.smtp_user,self.smtp_passwd)

   def close(self):
      self.imap.close()
      self.imap.logout()
      if self.smtp_enabled:
         self.smtp.quit()

   def fetch(self):
      self.lst_msg = []
      # fetching all new incoming mails
      self.imap.check()
      typ, data = self.imap.search(None, 'Unseen', '(FROM "' + self.mail_from + '")')
      for num in data[0].split():
         typ, data = self.imap.fetch(num, '(RFC822)')
         msg = email.message_from_string(data[0][1])
         self.lst_msg.append(msg)
         self.imap.store(num, '+FLAGS', '\\Seen')

   def add_all(self):
      for msg in self.lst_msg:
         if msg['Subject'] == self.mail_subject:
            self.add_entry(msg)

   def add_entry(self,msg):
      # we need a temp file
      tmp_file = tempfile.NamedTemporaryFile()
      # fetch the mail text
      lines = email.iterators.body_line_iterator(msg)
      for l in lines: tmp_file.write(l)
      tmp_file.flush()
      # call nb
      os.system("nb import entry " + tmp_file.name + " --no-preview")
      if self.smtp_enabled:
         # send mail to say that all is ok
         msg = MIMEText("Entry added")
         msg.set_charset('utf-8')
         msg['Subject'] = 'Entry added'
         msg['From'] = self.mail_me
         msg['To'] = self.mail_from
         self.smtp.sendmail(msg['From'], msg['To'], msg.as_string())

   def getdelay(self):
      return self.delay

m = Mail2nb()
m.connect()
while True:
   m.fetch()
   m.add_all()
   time.sleep(m.getdelay()*60)
m.close()
